from sqlalchemy import func

from gwlib.models import ModelJson
from . import db


class Zone(db.Model, ModelJson):
    __tablename__ = "gw_zone"

    zone_id = db.Column(db.Integer, primary_key=True)
    course_id = db.Column(db.Integer, db.ForeignKey('gw_golf_course.course_id'), nullable=False)
    code = db.Column(db.String(length=3), nullable=True)
    alias = db.Column(db.String(length=45), nullable=False)
    note_required = db.Column(db.Boolean, default=False, nullable=False)
    hole_linked = db.Column(db.Boolean, default=False, nullable=False)
    deleted = db.Column(db.Boolean, default=False, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)




from gwlib.base.base_service import BaseService
from gwlib.dao import BaseAlertDAO


class BaseAlertService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = BaseAlertDAO()

    def get_by_alert_id(self, alert_id):
        return self.dao.get(alert_id=alert_id).to_json()



import datetime

from . import default


class Config(default.Config):
    """
        development config
    """
    DEBUG = True
    JWT_ACCESS_TOKEN_EXPIRES = datetime.timedelta(days=15)
    SQLALCHEMY_DATABASE_URI = "mysql+pymysql://admin:groundworx$20@gwdb.cdwxp72nniac.us-east-1.rds.amazonaws.com" \
                              "/groundworx".format(
    )
    APP_HOST = "http://test-bucket-gw.s3-website-us-west-1.amazonaws.com/recover-password"
    BUCKET_NAME = 'groundworx-media'




from sqlalchemy import func
from sqlalchemy.orm import relationship

from gwlib.models import ModelJson
from . import db


class ZoneLocation(db.Model, ModelJson):
    __tablename__ = "gw_zone_location"

    location_id = db.Column(db.Integer, primary_key=True)
    hole_type_id = db.Column(db.Integer, db.ForeignKey('gw_zone.zone_id'), nullable=False)
    code = db.Column(db.String(length=45), nullable=True)
    alias = db.Column(db.String(length=45), nullable=False)
    deleted = db.Column(db.Boolean, default=False, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)




from sqlalchemy import func

from gwlib.models import db
from .model_json import ModelJson


class Resource(db.Model, ModelJson):
    __tablename__ = "gw_resource"

    resource_id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(25))
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)



class ConstErrors:
    FieldRequired = "0001"
    FieldError = "0002"


"""
Authrentication and permission Exceptions
"""


class UserNotAllowed(Exception):
    def __init__(self, user):
        # Call the base class constructor with the parameters it needs
        message = "Wrong User or password: {} ".format(user)
        super().__init__(message)
        # Now for your custom code...


class TypeUserNotDefined(Exception):
    def __init__(self, user):
        # Call the base class constructor with the parameters it needs
        message = "The user {}, has not type defined ".format(user)
        super().__init__(message)
        # Now for your custom code...


class PolicyRoleInvalid(Exception):
    def __init__(self, user):
        # Call the base class constructor with the parameters it needs
        message = "Role policy invalid to user: {} ".format(user)
        super().__init__(message)
        # Now for your custom code...


"""
Fields Exceptions

"""


class FieldRequired(Exception):
    def __init__(self, field, errors):
        # Call the base class constructor with the parameters it needs
        message = "The field {} is required".format(field)
        super().__init__(message)
        # Now for your custom code...
        self.errors = errors


class FieldError(Exception):
    def __init__(self, field, errors):
        # Call the base class constructor with the parameters it needs
        message = "The field {} is wrong".format(field)
        super().__init__(message)
        # Now for your custom code...
        self.errors = errors


class FieldNotInModel(Exception):
    def __init__(self, field, errors):
        # Call the base class constructor with the parameters it needs
        message = "Field {}, Not In Model".format(field)
        super().__init__(message)
        # Now for your custom code...
        self.errors = errors



import sys
import traceback

import sqlalchemy
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound

from gwlib.base.errors import UserNotAllowed
from gwlib.http.responses import HTTP_BAD_REQUEST, HTTP_CONFLICT, HTTP_RESPONSE, HTTP_SERVER_ERROR, HTTP_NOT_FOUND, \
    HTTP_NOT_PERMISSION


class BaseController:

    def build_response(self, method=None, **kwargs):
        """
        Method to call a Service function and return a Http Response
        :type kwargs: dict
        :type method: function
        """
        try:
            response = method(**kwargs)
        # authentication section
        except UserNotAllowed as e:
            traceback.print_exc(file=sys.stdout)
            print("ERROR", e)
            return HTTP_NOT_PERMISSION(e)
        except KeyError as e:
            print("ERROR", e)
            traceback.print_exc(file=sys.stdout)
            return HTTP_BAD_REQUEST(e)
        except IntegrityError as e:
            print("ERROR", e)
            traceback.print_exc(file=sys.stdout)
            return HTTP_CONFLICT(e)
        except NoResultFound as e:
            print("ERROR", e)
            traceback.print_exc(file=sys.stdout)
            return HTTP_NOT_FOUND("Not Results found")
        except MultipleResultsFound as e:
            print("ERROR", e)
            traceback.print_exc(file=sys.stdout)
            return HTTP_NOT_FOUND("Not found")
        except sqlalchemy.exc.InvalidRequestError as e:
            print("ERROR", e)
            traceback.print_exc(file=sys.stdout)
            return HTTP_BAD_REQUEST(e)
        except Exception as e:
            print("ERROR", e)
            traceback.print_exc(file=sys.stdout)
            return HTTP_SERVER_ERROR(e)

        return HTTP_RESPONSE(response)




from sqlalchemy import func
from sqlalchemy.orm import relationship

from gwlib.models import ModelJson
from . import db


class UserToken(db.Model, ModelJson):
    __tablename__ = "gw_user_token"

    token_id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, db.ForeignKey('gw_user.user_id'), nullable=False)
    token = db.Column(db.String(length=40), nullable=False)
    is_active = db.Column(db.Boolean, default=True, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now())

    user = relationship("User")



from gwlib.base.base_service import BaseService
from gwlib.dao.sensor_dao import BaseSensorDAO


class BaseSensorService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = BaseSensorDAO()

    def get_by_sensor_id(self, sensor_id):
        return self.dao.get(sensor_id=sensor_id).to_json()



import datetime
import os

from . import default


class Config(default.Config):
    """
        development config
    """
    DEBUG = True
    JWT_ACCESS_TOKEN_EXPIRES = datetime.timedelta(days=15)
    SQLALCHEMY_DATABASE_URI = "mysql+pymysql://{}:{}@{}/{}".format(
        os.environ.get('MYSQL_USER', "groundworx"),
        os.environ.get('MYSQL_PASS', "groundworx"),
        os.environ.get('MYSQL_HOST', "mysql"),
        os.environ.get('MYSQL_DB', "groundworx"),
    )
    APP_HOST = "http://test-bucket-gw.s3-website-us-west-1.amazonaws.com/recover-password"



from sqlalchemy.orm import relationship

from . import db
from gwlib.models.user_type_menu import user_type_menu
from .model_json import ModelJson


class UserType(db.Model, ModelJson):
    __tablename__ = "gw_user_type"

    type_id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(length=100), nullable=False)

    users = relationship("User", back_populates="user_type")
    user_types_policies = relationship("UserTypePolicy", lazy='dynamic')
    menus = db.relationship(
        "Menu",
        secondary=user_type_menu,
        passive_deletes=True,
        backref="user_menus")


import sys
import traceback
from functools import wraps

from sqlalchemy.orm.exc import NoResultFound

from gwlib.base.errors import PolicyRoleInvalid, TypeUserNotDefined
from gwlib.models import GolfCourse
from gwlib.models import User
from gwlib.services.base_policy_service import BasePolicyService
from gwlib.services.base_user_service import BaseUserService

try:
    from flask import _app_ctx_stack as ctx_stack, request, current_app, jsonify, g
except ImportError:  # pragma: no cover
    from flask import _request_ctx_stack as ctx_stack
from flask_jwt_extended import verify_jwt_in_request, get_jwt_identity


def resource_by_role(resource_name=None):
    from gwlib.security import method_to_permission

    def resource_by_role_decorator(fn):
        @wraps(fn)
        def resource_by_role_innner(*args, **kwargs):
            base_policy_service = BasePolicyService()
            base_user_service = BaseUserService()
            verify_jwt_in_request()
            current_user = get_jwt_identity()
            type_id = current_user.get("type_id")
            user_id = current_user.get("user_id")
            user = base_user_service.get_by_user_id(user_id=user_id)
            print("user", user)
            try:
                resource_permission = base_policy_service.get_resource_permissions(type_id, resource_name)
                print("resource", resource_permission)
                permission = method_to_permission.get(request.method)
            except PolicyRoleInvalid as e:
                print("Error: PolicyRoleInvalid")
                return jsonify(error=True, msg=str(e)), 403
            except TypeUserNotDefined as e:
                print("Error: TypeUserNotDefined")
                return jsonify(error=True, msg=str(e)), 403
            except NoResultFound:
                print("Error: NoResultFound")
                return jsonify(error=True, msg="Invalid User"), 403
            except Exception as e:
                traceback.print_exc(file=sys.stdout)
                print("ERROR >>>>>>>", e)
                return jsonify(error=True, msg='User not Allowed'), 403

            if not resource_permission.get(permission):
                print("PERMISION")
                return jsonify(error=True, msg='User not Allowed'), 403

            g.user = user
            print("aqui")
            # login_user(user)

            return fn(*args, **kwargs)

        return resource_by_role_innner

    return resource_by_role_decorator


def has_access_to_course(fn):
    @wraps(fn)
    def has_access_to_course_inner(*args, **kwargs):
        from gwlib.security import method_to_permission
        user = g.user  # type: User
        golf_courses = user.golf_courses  # type: [GolfCourse]
        path = request.path
        path_splited = path.split('/')
        microservice = path_splited[1]
        resource = path_splited[2]
        obj_id = path_splited[3] if len(path_splited) > 3 and path_splited[3].isnumeric() else None
        per_obj = path_splited[3] if len(path_splited) > 3 and not obj_id else None
        per_obj_id = path_splited[4] if len(path_splited) > 4 and path_splited[4].isnumeric() else None
        users = []
        holes = []
        golf_ids = []
        have_access = False
        for golf_course in golf_courses:
            golf_ids.append(golf_course.course_id)
            holes.append(golf_course.holes)
            users.extend(golf_course.users)

        permission = method_to_permission.get(request.method)
        json = request.json

        if resource == "user":
            # can access to course
            if request.method != "POST":
                if per_obj == "per_course":
                    if int(per_obj_id) in golf_ids:
                        have_access = True
                else:
                    # users can access
                    for user in users:
                        if user.user_id == int(obj_id):
                            have_access = True
            else:
                course_id = json.get("course_id")
                if course_id in golf_ids:
                    have_access = True

        elif resource == "hole":
            # can access to course
            if per_obj == "per_course":
                if int(per_obj_id) in golf_ids:
                    have_access = True
            # holes can access
            for hole in holes:
                if hole.hole_id == int(obj_id):
                    have_access = True

        elif resource == "golf_course":
            if int(obj_id) in golf_ids:
                have_access = True

        if not have_access:
            return jsonify(error=True, msg='User not Allowed'), 403

        return fn(*args, **kwargs)

    return has_access_to_course_inner



from gwlib.base.base_dao import BaseDAO
from gwlib.base.errors import FieldRequired


class BaseService:
    required_fields = []

    def __init__(self):
        self.data = {}
        self.dao = BaseDAO()

    def validate(self):
        for key in self.required_fields:
            if key not in self.data:
                raise FieldRequired

    def __save(self):
        self.validate()
        self.before_save()
        response = self.dao.save(**self.data)
        self.after_save()
        return response

    def __update(self, data=None, filters=None):
        self.data = data
        return self.dao.update(data=self.data, **filters)

    def before_save(self):
        pass

    def after_save(self):
        pass

    def save(self, **data):
        self.data = data
        return self.__save()

    def update(self, **args):
        return self.__update(**args).to_json()

    def delete(self, **filters):
        return self.dao.delete(**filters)

    def filter(self, **filters):
        return self.dao.filter(**filters)



from flask import g
from sqlalchemy.orm import Query
from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound

from gwlib.base.errors import FieldNotInModel
from gwlib.utils.helper import Helper


class BaseDAO:
    include_logical_deleted = False
    __model = None

    def __init__(self, model=None):
        from gwlib.models import db
        self.session = db.session
        self.__model = model

    def change_model(self, model):
        self.model = model

    @property
    def model(self):
        return self.__model

    @model.setter
    def model(self, model):
        self.__model = model

    @property
    def queryset(self):
        """

        :rtype: Query
        """
        if not self.include_logical_deleted and hasattr(self.__model, 'deleted'):
            return self.model.query.filter_by(deleted=False)
        else:
            return self.model.query

    def save(self, **data):
        model = self.model(**data)
        self.session.add(model)
        self.session.commit()
        return model.to_json()

    def __get(self, **filters):
        try:
            query = self.queryset.filter_by(**filters)
            obj = query.one_or_none()
            if obj is None:
                raise NoResultFound

        except MultipleResultsFound:
            raise MultipleResultsFound
        return obj

    def update(self, data=None, **filters):
        obj = self.__get(**filters)
        for field, value in data.items():
            if field == "password":
                value = Helper.set_crypt(value)
            if hasattr(obj, field):
                setattr(obj, field, value)
            else:
                raise FieldNotInModel(field, {})

        self.session.commit()
        return obj

    def delete(self, **filters):
        obj = self.__get(**filters)
        if hasattr(obj, "deleted"):
            obj.deleted = True
        self.session.commit()
        return obj

    def filter(self, **filters):
        query = self.queryset.filter_by(**filters)
        return query.all()

    def get(self, **filters):
        return self.__get(**filters)



from gwlib.base.base_service import BaseService
from gwlib.dao import BaseResourceDAO


class BaseResourceService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = BaseResourceDAO()

    def get_by_name(self, resource_name):
        return self.dao.get(name=resource_name)



from gwlib.base.base_service import BaseService
from gwlib.dao.user_policy_dao import UserPolicyDAO
from gwlib.services.base_resource import BaseResourceService


class BasePolicyService(BaseService):

    def __init__(self):
        super().__init__()
        self.dao = UserPolicyDAO()

    def get_policy(self, resource_id, type_id):
        return self.dao.get(resource_id=resource_id, type_id=type_id)

    def get_resource_permissions(self, type_id, resource_name):
        resource_service = BaseResourceService()
        resource = resource_service.get_by_name(resource_name=resource_name)
        print("aqui resource perm", resource)
        return self.dao.get_permissions_by_resource(type_id=type_id,
                                                    resource_id=resource.resource_id).to_json()










from sqlalchemy import Table

from . import db

user_golf_course = Table('gw_user_golf_course',
                         db.Model.metadata,
                         db.Column('course_id', db.Integer, db.ForeignKey('gw_golf_course.course_id'), nullable=False),
                         db.Column('user_id', db.Integer, db.ForeignKey('gw_user.user_id'), nullable=False),
                         db.Column('deleted', db.Boolean, default=False, nullable=False)
                         )



import json
from json import JSONDecodeError

from flask import jsonify

def build_json_error(error):
    return {
        "success": False,
        "error": str(error)
    }


def _valid_response(response):
    try:
        json.loads(response)
    except JSONDecodeError:
        raise JSONDecodeError


def HTTP_BAD_REQUEST(response):
    response = build_json_error(response)
    return jsonify(response), 400


def HTTP_NOT_PERMISSION(response):
    response = build_json_error(response)
    return jsonify(response), 401


def HTTP_RESPONSE(response):
    # _valid_response(response)
    return jsonify(response), 200


def HTTP_NOT_FOUND(response):
    return jsonify(response), 404


def HTTP_CONFLICT(response):
    response = build_json_error(response)
    return jsonify(response), 405


def HTTP_SERVER_ERROR(response):
    response = build_json_error(response)
    return jsonify(response), 500












from .base_policy_service import BasePolicyService
from .base_resource import BaseResourceService
from .base_user_service import BaseUserService
from .base_course_service import BaseGolfCourseService
from .base_holes_service import BaseHolesService
from .base_sensor_service import BaseSensorService
from .base_alert_service import BaseAlertService
from .base_menu_service import BaseMenuService
from .base_report_service import BaseReportService



from sqlalchemy import func
from sqlalchemy.orm import relationship

from gwlib.models import ModelJson
from . import db


class Sensor(db.Model, ModelJson):
    __tablename__ = "gw_sensor"

    sensor_id = db.Column(db.Integer, primary_key=True)
    hole_id = db.Column(db.Integer, db.ForeignKey('gw_hole.hole_id'))
    quadrant_id = db.Column(db.Integer, db.ForeignKey('gw_zone_quadrant.quadrant_id'))
    zone_id = db.Column(db.Integer, db.ForeignKey('gw_zone.zone_id'))
    location_id = db.Column(db.Integer, db.ForeignKey('gw_zone_location.location_id'))
    meid = db.Column(db.String(length=18), nullable=False)
    serial_number = db.Column(db.String(length=18), nullable=False)
    latitude = db.Column(db.DECIMAL(13, 10), nullable=True)
    longitude = db.Column(db.DECIMAL(13, 10), nullable=True)
    status = db.Column(db.String(10), nullable=False)
    name = db.Column(db.String(20), nullable=False)
    location_note = db.Column(db.String(120), nullable=False)
    location_desc = db.Column(db.String(150), nullable=False)
    deleted = db.Column(db.Boolean, default=False, nullable=False)
    time_created = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)
    time_updated = db.Column(db.DateTime(timezone=True), server_default=func.now(), nullable=False)

    hole = relationship("Hole")
